<?php
/* ----------------------
   glPage class is a container for a html page.
   with support for blocks, resources (js/css), templates
 */
class glPage {
public           $name           = '';                          // name of the node in lowercase
protected        $localNSI       = '';                          // local style NameSpace Identifier (a prefix)
protected        $layout         = array();                     // a list of consecutive containers; used for sequencing them in the final node
protected        $skins          = array();                     // registered skins as scoped styles
protected        $resources      = array();                     // registered resources, will be injected in head or foot 
protected        $templates      = array();                     // all instantiated templates, will be finished on render
protected        $snippets       = array();                     // collection of all content snippets, ready to go into a Template
protected        $messages       = array();                     // all caught messages to display

public           $noGlue         = false;
protected        $glue           = array('fieldset', 'legend');
protected static $resourceCount  = 100;

protected static $pageOutline    = '';                          // string containing outline (layout of root page)
public    static $pageTitle      = '';
protected static $cssSelectorNSI = 'gl';                        // global style NameSpace Identifier (a prefix)
protected static $nodes          = array();                     // all instantiated node objects, indexed on name

private function      __construct($name)            { $this->name  = strtolower($name); $this->localNSI = self::$cssSelectorNSI; }             // initialize a new node Object
public  function     setPageTitle($title)           { self::$pageTitle  = $title; }                                                            // pageTitle
public  function        setLayout($string)          { $this->layout = $this->buildLayout($string, $this->localNSI); }                          // set a layout list and build

public  function setCssSelectorNS($prefix)          { self::$cssSelectorNSI = $prefix; }                                                       // glue a global nameSpace
public  function       setStyleNS($prefix)          { $this->localNSI = $prefix; }                                                             // glue a local NameSpace
public  function       addSnippet($name, $content)  { $this->snippets[$name] = $content; }                                                     // add a snippet to a layout container
public  function       getSnippet($name)            { return $this->snippets[$name]; }                                                         // get all snippets from a layout container
public  function          getPage($parentName)      { return self::getNode($parentName); }                                                     // retrieve a Page Object (or create one)

private function    setJavascript($id, $what)       { $GLOBALS['_SCRIPTS']->setJavaScriptFile($id, $what, false,  self::$resourceCount++);  }  // js resource in header seq 100++
private function    addJavascript($id, $what)       { $GLOBALS['_SCRIPTS']->setJavaScriptFile($id, $what, true,   self::$resourceCount++);  }  // js resource in footer seq 100++
private function    addStylesheet($id, $what)       { $GLOBALS['_SCRIPTS']->setCssFile($id, $what, true, array(), self::$resourceCount++); }   // css resource in head seq 100
public  function       setLibrary($which)           { $GLOBALS['_SCRIPTS']->setJavaScriptLibrary($which, false); }                             // in head
public  function       addLibrary($which)           { $GLOBALS['_SCRIPTS']->setJavaScriptLibrary($which, true); }                              // in footer
public  function        addScript($where, $content) { $GLOBALS['_SCRIPTS']->setJavaScript($content, true, false); }                            // wrap a script, in head
public  function         addStyle($where, $content) { $GLOBALS['_SCRIPTS']->setCSS($content); }                                                // wrap a stylesheet
public  function          addSkin($name, $class)    { $this->skins[$name][] = $class; }                                                        // add a skin class to a layout container
public  function          setSkin($name, $file)     { $this->resources[$name]['skin'] = $file; }                                               // set resource needed for a skin

public  function setOutline($outline)
{
    if (empty(self::$pageOutline)) {
        self::$pageOutline = $outline;
    } else {
        $staticOutline = self::$pageOutline;
        throw(new Exception('#glPage.Outline#' . "Duplicate [{$outline}] invalidates [{$staticOutline}]"));
    }
}
private function buildLayout($layout, $myNSI) 
{
    $proto = array();
    foreach (split(',', $layout) as $string)
        {   list($name, $id, $class) = split(':', $string);
            $class   = (empty($class)) ? $myNSI .'-'. $name : $class;   // @todo: use a class constant
            $id      = (empty($id))    ? $myNSI .'_'. $name : $id;      // @todo: use a class constant
            $proto[] = array('name' => $name, 'id' => $id, 'class' => $class, 'namespace' => $myNSI, 'debug' => true); // debug false not implemented
        }
    return $proto;
}

public  function addResource($id, $what)
{
    $type = substr($what, strrpos($what, '.', -1));
//  $this->addContent('debug', "insert $what with id $id as type $type \n");
    switch ($type) {
        case ''      :  break;
        case '.js'   :  $this->addJavascript($id, $what);
                        break;
        case '.css'  :
        case '.css3' :  $this->addStylesheet($id, $what);
                        break;
        default      :  throw(new Exception('#glPage.Resources#' . "Invalid resource type [{$type}]"));
    }
} 
public  function setResource($id, $what)
{
    $type = substr($what, strrpos($what, '.', -1));
//  $this->addContent('debug', "insert $what with id $id as type $type \n");
    switch ($type) {
        case '.js'   :  $this->setJavascript($id, $what);
                        break;
        case '.css'  :
        case '.css3' :  $this->addStylesheet($id, $what);
                        break;
        default      :  throw(new Exception('#glPage.Resources#' . "Invalid resource type [{$type}]"));
    }
} 

public function getNode($parentName) 
{
    $name  = strtolower($parentName);
    if (! isset(self::$nodes[$name])) {
        $node = new glPage($name);
        self::addNode($node);
    }
    if (! self::$nodes[$name] instanceOf glPage) {
        throw(new Exception('#glPage.Failure#' . "Invalid node on [{$parentName}]"));
    }
    return self::$nodes[$name];
}

protected function addNode(glPage $node) 
{
    $name = $node->name;
    if (isset(self::$nodes[$name])) {
        $names = implode(',', array_keys(self::$nodes));
        throw(new Exception('#glPage.Failure#' . "Constructed node on [{$name}] is duplicated in [{$names}]"));
    }
    self::$nodes[$name] = $node;
}

public function addMessage($msg, $context)
{
    $this->messages[$context][] = COM_showMessageText($msg, 'Message from '.$this->name);         // @todo: return COM_showMessage($msg, $context);
}

public function getMessages($context = '')
{
    if (empty($context)) { return $this->messages; }
    else                 { return $this->messages[$context]; }
}

public function getMessagesContent($context = '') // @todo: wrap a message with a context glue
{
    $messages = $this->getMessages($context);
    $content  = '';
    if (!empty($messages)) {
        foreach ($messages as $seq => $message) {
            $content .= $message;
        }
    }
    return $content;
}

public function setContent($what, $snippet, $append = false)
{
    if ($append) { $this->addContent($what, $snippet); }
    else         { $this->addSnippet($what, $snippet); }
}

public function addContent($what, $snippet)
{
    $this->addSnippet($what, $this->getSnippet($what) . $snippet);
}
/*  1) snippets, 2) node, 3) template
    @todo: check for recusrsion
 */
public function getContent($what = '')
{
    $content = '';
    foreach ($this->layout as $id => $page) {
        $that     = $page['name'];
        $content .= "\n<!-- [{$what}] [{$that}] [ns:{$page['namespace']}]-->\n";
        if (empty($what) or $what == $that or $what == $this->name) {
            $content   .= $this->glueBlock('start', $what, $page);
            $content   .= $this->getMessagesContent($that);
            $headerVar  = $this->getSnippet($that);
            $myContent  = $headerVar;
            if (isset(self::$nodes[$that])) { 
//              $myContent .= $this->getPageContent($that); 
                $bodyVar    = self::$nodes[$that]->getContent($that);
                $myContent .= $bodyVar;
            }
            if (count($this->templates[$that]) > 0) {
                foreach ($this->templates[$that] as $seq => $template) {
                    $template->set_var("header_$that", $headerVar);
                    $template->set_var("body_$that", $bodyVar);
                    $content .= $template->parse('glPage', "page_$that");
                }
            } else {
                $content .= $myContent;
            }
            $content .= $this->glueBlock('end', $what, $page);
        }
    }
    return $content;
}

private function glueBlock($how, $what, $page)
{
    if (empty($page['id']) or $this->noGlue) { return ''; }
    $that = $page['name'];
    if (substr($that, -3) == 'fix' ) { return ''; }
    switch ($how) {
        case 'start' :
            if ($page['debug']) { $title = "title='{$that} of {$what}'"; }
            else                { $title = ''; }
            if (isset($this->skins[$what])) { 
                $class = ' ' . implode(' ', $this->skins[$what]); 
                if (isset($this->resources[$what]['skin'])) { $this->addResource('skin', $this->resources[$what]['skin']); }
            } else {
                $class = '';
            }
            $content = "<div id='{$page['id']}' class='{$page['class']}{$class}' {$title} >";
            if ($page['debug'] and $that != 'geeklog') { $content .= "<fieldset id='{$page['id']}-fs'><legend>{$that}</legend>"; }
            return $content;
            break;
        case 'end'   :
            if ($page['debug'] and $that != 'geeklog') { $content = "</fieldset>"; }
            else                                       { $content = ''; }
            $content .= "</div>";
            return $content;
            break;
        default      :
            throw(new Exception('#glPage.Blocks#' . "Invalid glue specified [{$how}]"));
    }
    return '';
}

public function addTemplate($what, $file, $root = '.')
{
    $template = new Template($root);
    $template->set_file("page_$what", $file);
    $this->templates[$what][] = $template;
    return $template;
}

public function hookTemplate($purpose='', $context='')
{
    $this->assignTemplate($purpose, $context);
}

protected function assignTemplate($purpose = 'default', $context ='')
{
    if (isset($this->template)) { return; }

    if (empty($purpose))        { $purpose = 'default'; }
    if (empty($context))        { $context = self::$compliance; }

    if (empty($this->tRoot))    { $tPath  = $this->getTemplateRoot($context, $purpose); }
    else                        { $tPath  = self::$layout . $this->tRoot; }

    if (!isset($this->template)) { $this->template = new Template($tPath); }
    $this->template->set_file($this->tFiles);
}

/*
 *  acts as facade to proxy @todo 
 */
protected function getTemplateRoot($context = 'public', $purpose ='default')
{
    global $_CONF;
    $plugin     = $this->name;
    $glPageConf = '_' .strtoupper($plugin). '_CONF'; // @todo find the plugin Name or call canvas
    global $$glPageConf;
    $glPageConf = $$glPageConf;

    if ($context === 'admin') {
        $configured_tDir = $glPageConf['templates_admin'];
        $fallback = 'admin';
    } else {
        $configured_tDir = $glPageConf['templates_user'];
        $fallback = 'public';
    }

    switch ($configured_tDir) {
        case 'theme' :
            $tPath = $_CONF['path_layout'] . $glPageConf['themespath'] .$context;
            if ($context <> 'admin')     { $tPath .= '/' . $purpose; }
            if (!is_dir($tPath)) {
                $tPath = $_CONF['path'] . 'plugins/' .$plugin. '/templates/' .$fallback;
                if ($context <> 'admin') { $tPath .= '/default'; }
            }
            break;
        case 'custom' :
            $tPath = $_CONF['path'] . 'plugins/' .$plugin. '/custom/templates/' .$context;
            if ($context <> 'admin')     { $tPath .= '/' . $purpose; }
            if (!is_dir($tPath)) {
                $tPath = $_CONF['path'] . 'plugins/' .$plugin. '/templates/' .$fallback;
                if ($context <> 'admin') { $tPath .= '/default'; }
            }
            break;
        case 'standard':
        default        :
            $tPath = $_CONF['path'] . 'plugins/' .$plugin. '/templates/' .$context;
            if ($context <> 'admin')     { $tPath .= '/' . $purpose; }
            if (!is_dir($tPath)) {
                $tPath = $_CONF['path'] . 'plugins/' .$plugin. '/templates/' .$fallback;
                if ($context <> 'admin') { $tPath .= '/default'; }
            }
    }
    return $tPath;
}

/* @TODO check COM_ and names */
public function renderPage($outline = '' , $pageContent = '', $custom = '' )
{
    global $_CONF;

//    @todo: $glPageConf = '_'.strtoupper($this->pi_name)."_CONF";
//    @todo: global $$glPageConf;
//    @todo: $glPageConf = $$glPageConf;

    $information = array();
    if  ($outline == '_admin' OR $outline == '')    { $layout = $glPageConf['layout'.$outline]; } 
    else                                            { $layout = $outline; }
    if (!empty($custom))                            { $information['custom'] = $custom; }

    switch ($layout) {
        case 'standard':
            $information['what']            = 'menu';
            $information['rightblock']      = false;
            break;
        case 'leftrightblocks':
            $information['what']            = 'menu';
            $information['rightblock']      = true;
            break;
        case 'rightblocks':
            $information['what']            = 'menu';
            $information['rightblock']      = true;
// @todo:   $display .= COM_siteFooter(true, $custom);
            break;
        case 'leftblocks':
            $information['what']            = 'menu';
            $_CONF['show_right_blocks']     = 0;
            $_CONF['left_blocks_in_footer'] = 0;
            $information['rightblock']      = false;
            break;
        case 'noblocks':
            $information['what']            = 'none';
            $information['rightblock']      = false;
            $_CONF['show_right_blocks']     = 0;
            $_CONF['left_blocks_in_footer'] = 0;
            break;
        case 'blankpage':
            break;
        default:
    }

    $information['pagetitle'] = self::$pageTitle;

    $display = $pageContent . self::getPageContent();

    if (COM_versionCompare(VERSION, "2.0.0",  '>=')) {
        $display  = COM_createHTMLDocument($display, $information);
    }else{
        $display  = COM_siteHeader($information['what'], $information['pagetitle']) . $display;
        $display .= COM_siteFooter($information['rightblock']);
    }
    if (empty($pageContent)) { COM_output($display); exit; }
    else                     { return $display; }
}

public function getOutline($what = '')
{
    if (empty($what))   { $proto = self::buildLayout(self::$pageOutline, self::$cssSelectorNSI); }
    else                { $proto = $this->layout; }
    if (!count($proto) ) { $keys = '_'; }
    foreach ($proto as $seq => $page) {
        $that = $page['name'];
        if (empty($what) or $what == $that or $what == $this->name) {
            $keys .= $comma . $that . ':';
            if (isset(self::$nodes[$that])) { 
                $keys .= self::$nodes[$that]->getOutline($that); 
            } else {
                $keys .= '?';
            }
            $comma = ',';
        }
    }
    return $keys;
}

public function getPageContent($what = '')
{
    if (empty($what))   { $proto = self::buildLayout(self::$pageOutline, self::$cssSelectorNSI); }
    else                { $proto = $this->layout; }
    $content   = '';
    foreach ($proto as $seq => $page) {
        $that = $page['name'];
        if (empty($what) or $what == $that or $what == $this->name) {
            $content .= "\n<!-- node:".$that." ns:".$page['namespace']." -->\n" . self::glueBlock('start', $that, $page);
            if (isset(self::$nodes[$that])) { 
                $content .= self::$nodes[$that]->getContent($that); 
            }
            $content .= self::glueBlock('end', $that, $page) . "\n<!-- end:" .$that. " -->\n";
        }
    }
    return $content;
}

/*
 * fix new lines in raw text
 */
public function fixNewLines($text) 
{
    return str_replace(array("\r\n","\n\r","\r","\n"), '<br'.XHTML.'>', $text);
}

/*
 * scan for autolinks and fix them in raw text
 */
public function fixAutoLink($text) 
{
    $patterns     = array("/(https?|ftp)(:\/\/[[:alnum:]\+\$\;\?\.%,!#~*\/:@&=_-]+)/i");
    $replacements = array("<a href=\"\\1\\2\" target=\"_blank\">\\1\\2</a>");
    return preg_replace($patterns, $replacements, $text);
}

} // end of glPage
 ?>